@@ -5,7 +5,7 @@ use std::sync::Arc;
 use rayon::prelude::*;
 use swc_common::{collections::AHashSet, pass::Repeated, util::take::Take, DUMMY_SP, GLOBALS};
 use swc_ecma_ast::*;
-use swc_ecma_utils::collect_decls;
+use swc_ecma_utils::{collect_decls, undefined};
 use swc_ecma_visit::{noop_visit_mut_type, VisitMut, VisitMutWith, VisitWith};
 use tracing::{span, Level};
 
@@ -51,6 +51,7 @@ pub(crate) fn pure_optimizer<'a>(
         data,
         ctx: Ctx {
             force_str_for_tpl,
+            top_level: true,
             ..Default::default()
         },
         changed: Default::default(),
@@ -261,6 +262,30 @@ impl VisitMut for Pure<'_> {
             e.visit_mut_children_with(&mut *self.with_ctx(ctx));
         }
 
+        if self.options.unused {
+            if let Expr::Unary(UnaryExpr {
+                span,
+                op: op!("void"),
+                arg,
+            }) = e
+            {
+                if !arg.is_lit() {
+                    self.ignore_return_value(
+                        arg,
+                        DropOpts {
+                            drop_global_refs_if_unused: true,
+                            drop_zero: true,
+                            drop_str_lit: true,
+                        },
+                    );
+                    if arg.is_invalid() {
+                        *e = *undefined(*span);
+                        return;
+                    }
+                }
+            }
+        }
+
         self.eval_trivial_values_in_expr(e);
 
         self.remove_invalid(e);
@@ -422,9 +447,14 @@ impl VisitMut for Pure<'_> {
     fn visit_mut_module_items(&mut self, items: &mut Vec<ModuleItem>) {
         self.bindings = Some(Arc::new(collect_decls(items)));
 
-        self.visit_par(items);
+        let ctx = Ctx {
+            top_level: true,
+            ..self.ctx
+        };
 
-        self.handle_stmt_likes(items);
+        self.with_ctx(ctx).visit_par(items);
+
+        self.with_ctx(ctx).handle_stmt_likes(items);
     }
 
     fn visit_mut_new_expr(&mut self, e: &mut NewExpr) {
@@ -598,6 +628,8 @@ impl VisitMut for Pure<'_> {
             }
         }
 
+        self.eval_free_iife(s);
+
         self.loop_to_for_stmt(s);
 
         self.drop_instant_break(s);
@@ -636,10 +668,14 @@ impl VisitMut for Pure<'_> {
                 }
             }
         }
+        let ctx = Ctx {
+            top_level: false,
+            ..self.ctx
+        };
 
-        self.visit_par(items);
+        self.with_ctx(ctx).visit_par(items);
 
-        self.handle_stmt_likes(items);
+        self.with_ctx(ctx).handle_stmt_likes(items);
 
         items.retain(|s| !matches!(s, Stmt::Empty(..)));
 
